
; disassembly of SDC 1B disk controller. 05/02/21 Lee Davison.

; ? where unsure or just plain don't know, most of this is guesswork, some of that
; probably wrong

; disks are double sided, 80 track, 16 sector, 256 bytes per sector for a total of
; 640KB per disk. Two drives are supported.

; commands from the host to the controller are all 4 bytes long
; bit 0 of the command byte is the drive select bit. 0 = drive 0, 1 = drive 1

; command	byte 1	byte 2	byte 3	comments
; -------	------	------	------	--------
; $90		 xx		 xx		 xx		returns unknown command error
; $88		 xx		 xx		 xx		returns unknown command error
; $80		 xx		 xx		 xx		seek to track zero
; $98		 xx		 xx		 xI		format disk
; $D0		 xx		 xx		 xx		returns unknown command error
; $C8		 0T		 ts		 ??		write sector
; $C0		 0T		 ts		 ??		read sector

; x = ignored
; T = track high nibble, if > $04 then side 1
; t = track low nibble
; s = sector
; 0 = must be 0
; ? = dunno yet
; I = interleave

; returned error codes

; code	error					; code	error
; ----	-----					; ----	-----
; $21		command parameter			; $20		unknown command
; $1A		not 16 sectors written		; $15		seek track
; $14		seek sector				; $11		CRC
; $0B		not 256 bytes written		; $0A		not 256 bytes read
; $09		data read				; $08		data write
; $07		timeout				; $06		seek track zero
; $04		drive not ready			; $03		write protect

; $00		no error

; notes for R6541AQ ....
; the stack pointer is 8 bits and ranges from $007F down to $0040

LAB_00	= $00			; port A
					; bit	function
					; ---	--------
					; 7	0 = drive 0 select
					; 6	0 = drive 1 select
					; 1	wd INTRQ
					; 0	wd DRQ

LAB_01	= $01			; port B
					; bit	function
					; ---	--------
					; 4	??
					; 3	1 = motor on
					; 1	??

LAB_02	= $02			; port C
					; bit	function
					; ---	--------
					; 6	??, goes to host port but not used

LAB_10	= $10			; clear interrupt flag register
LAB_11	= $11			; interrupt flag register
					; bit	function	1 = interrupt
					; ---	--------	-----------
					; 5	counter
					; 4	external IRQ 2
					; 3	external IRQ 1
					; 2	internal IRQ
					; 1	PA0 -ve edge detect
					; 0	PA0 +ve edge detect

LAB_12	= $12			; interrupt enable register
					; bit	function	1 = enabled
					; ---	--------	-----------
					; 5	counter
					; 4	external IRQ 2
					; 3	external IRQ 1
					; 2	internal IRQ
					; 1	PA0 -ve edge detect
					; 0	PA0 +ve edge detect

LAB_14	= $14			; mode control register
;LAB_18	= $18			; upper counter/upper latch A
LAB_19	= $19			; lower counter#/upper latch*# A
					; * and start counter
					; # and clear flag
LAB_1A	= $1A			; lower counter/lower latch A
LAB_1C	= $1C			; host buffer
LAB_1E	= $1E			; host status register
					; bit	function
					; ---	--------
					; 7	1 = busy, 0 = ready
					; 6	1 = Rx, 0 = Tx
					; 5	1 = data, 0 = command
					; 2	last RS written by host
					; 1	output data register full
					; 0	input data register full

LAB_40	= $40			; WDxxxx status register copy
LAB_41	= $41			; buffer pointer low byte
LAB_42	= $42			; buffer pointer high byte

; WDxxxx floppy disk controller

LAB_F080	= $F080		; command/status register
LAB_F081	= $F081		; track register
LAB_F082	= $F082		; sector register
LAB_F083	= $F083		; data register

; RAM

LAB_F200	= $F200		; command index
LAB_F201	= $F201		; command drive number
LAB_F202	= $F202		; command byte 1
LAB_F203	= $F203		; command byte 2
LAB_F204	= $F204		; command byte 3

LAB_F206	= $F206		; current interleave
LAB_F207	= $F207		; report byte
LAB_F20C	= $F20C		; current drive number
LAB_F210	= $F210		; current track
LAB_F211	= $F211		; current sector
LAB_F212	= $F212		; current side
LAB_F213	= $F213		; max drive+1
LAB_F214	= $F214		; disk command store
LAB_F215	= $F215		; command byte 3 copy, never read
LAB_F216	= $F216		; active drive
LAB_F217	= $F217		; other drive track
LAB_F220	= $F220		; countdown value
LAB_F240	= $F240		; external interrupt vector low byte
LAB_F241	= $F241		; external interrupt vector high byte
LAB_F242	= $F242		; PA0 -ve edge interrupt vector low byte
LAB_F243	= $F243		; PA0 -ve edge interrupt vector high byte
LAB_F244	= $F244		; internal interrupt vector low byte
LAB_F245	= $F245		; internal interrupt vector high byte
LAB_F246	= $F246		; counter interrupt vector low byte
LAB_F247	= $F247		; counter interrupt vector high byte
LAB_F248	= $F248		; command vector address low byte
LAB_F249	= $F249		; command vector address high byte
LAB_F24A	= $F24A		; interrupt code register, never read ??

LAB_F300	= $F300		; 256 byte buffer to LAB_F3FF

; the EPROM is actually a 4k device but only 2k is addressed, both 2k halves seem to
; contain identical code. only the upper 2k is here

	.ORG	$F800

; set up vectors

LAB_F800
	LDA	#<LAB_F829		; get external interrupt code low byte
	STA	LAB_F240		; save external interrupt vector low byte
	LDA	#>LAB_F829		; get external interrupt code high byte
	STA	LAB_F241		; save external interrupt vector high byte
	LDA	#<LAB_F830		; get host bus interrupt code low byte
	STA	LAB_F242		; save PA0 -ve edge interrupt vector low byte
	LDA	#>LAB_F830		; get host bus interrupt code high byte
	STA	LAB_F243		; save PA0 -ve edge interrupt vector high byte
	LDA	#<LAB_F83B		; get internal interrupt code low byte
	STA	LAB_F244		; save internal interrupt vector low byte
	LDA	#>LAB_F83B		; get internal interrupt code high byte
	STA	LAB_F245		; save internal interrupt vector high byte
	LDA	#<LAB_F84A		; counter underflow interrupt code low byte
	STA	LAB_F246		; save counter interrupt vector low byte
	LDA	#>LAB_F84A		; counter underflow interrupt code high byte
	STA	LAB_F247		; save counter interrupt vector high byte
	RTS

; external interrupt code

LAB_F829
	LDA	#$80			; set code $80
	STA	LAB_F24A		; save interrupt code ??
	BMI	LAB_F853		; restore A and exit interrupt

; host bus interrupt code

LAB_F830
	RMB1	LAB_12		; clear PA0 -ve edge detect enable
	RMB1	LAB_10		; clear PA0 -ve edge detect flag
	LDA	#$81			; set code $81
	STA	LAB_F24A		; save interrupt code ??
	BMI	LAB_F853		; restore A and exit interrupt

; internal interrupt code

LAB_F83B
	RMB2	LAB_12		; clear internal IRQ enable
	LDA	#$00			; clear A
	STA	LAB_1E		; clear host status register
	LDA	LAB_1C		; read byte from host
	LDA	#$82			; set code $82
	STA	LAB_F24A		; save interrupt code ??
	BMI	LAB_F853		; restore A and exit interrupt

; counter underflow interrupt code

LAB_F84A
	RMB5	LAB_12		; clear counter interrupt enable
	LDA	#$83			; set counter high byte, 8.4ms
	STA	LAB_19		; save counter high byte and start counter
	STA	LAB_F24A		; save interrupt code ??

; restore A and exit interrupt

LAB_F853
	PLA				; restore A
	RTI

; initiate LAB_F220 countdown

LAB_F855
	LDA	#<LAB_FB71		; get decrement delay count vector low byte
	STA	LAB_F246		; save counter interrupt vector low byte
	LDA	#>LAB_FB71		; get decrement delay count vector low byte
	STA	LAB_F247		; save counter interrupt vector high byte
	LDA	#$44			; set counter low byte
	STA	LAB_1A		; save counter low byte
	LDA	#$4E			; set counter high byte, 5ms
	STA	LAB_19		; save counter high byte and start counter
	SMB5	LAB_12		; set counter interrupt enable
	RTS

; select current drive

LAB_F86A
	LDA	#$03			; mask 0000 00xx
	AND	LAB_F20C		; get current drive number
	TAX				; copy to index
	CMP	LAB_F216		; compare with active drive
	BEQ	LAB_F884		; branch if current is active

	STX	LAB_F216		; save current as active drive
	LDY	LAB_F081		; get WDxxxx track register
	LDA	LAB_F217		; get other drive track position
	STA	LAB_F081		; set WDxxxx track register
	STY	LAB_F217		; save other drive track position
LAB_F884
	LDA	LAB_F8B5,X		; get drive select pattern
	STA	LAB_00		; set drive select pattern
	LDA	#$64			; set for 100d counts
	STA	LAB_F220		; save countdown value
	JSR	LAB_F855		; initiate LAB_F220 countdown
	CLI				; enable interrupts
LAB_F892
	LDA	LAB_F080		; get WDxxxx status register
	BPL	LAB_F89F		; if ready go interrupt WDxxxx if busy

	LDA	LAB_F220		; get countdown value
	BNE	LAB_F892		; loop if not timed out

	LDX	#$04			; flag drive select error
	RTS

					; interrupt WDxxxx if busy
LAB_F89F
	RMB5	LAB_12		; clear counter interrupt enable
	LDX	#$01			; set delay count
	LDA	LAB_F080		; get WDxxxx status register
	AND	#$01			; mask 0000 000x, busy
	BEQ	LAB_F8B1		; branch if not busy

	LDA	#$D0			; force interrupt
	STA	LAB_F080		; set WDxxxx command register
	LDX	#$0F			; set delay count
LAB_F8B1
	DEX				; decrement count
	BNE	LAB_F8B1		; loop if more to do

	RTS

LAB_F8B5
	.byte	$7F,$BF,$FF,$FF	; drive control byte

; seek to track zero

LAB_F8B9
	JSR	LAB_F86A		; select current drive
	TXA				; copy error byte
	BNE	LAB_F8E8		; if error deselect drives and exit

	LDA	#$08			; seek track 00, head load, 3ms step
	STA	LAB_F080		; set WDxxxx command register
	LDA	#$C8			; set for 200d counts
	STA	LAB_F220		; save countdown value
	JSR	LAB_F855		; initiate LAB_F220 countdown
	LDX	#$07			; flag timeout error
LAB_F8CE
	LDA	LAB_F220		; get countdown value
	BEQ	LAB_F8E8		; if timed out deselect drives and exit

	LDA	LAB_F080		; get WDxxxx status register
	AND	#$01			; mask 0000 000x, busy
	BNE	LAB_F8CE		; loop if busy

	RMB5	LAB_12		; clear counter interrupt enable
	LDA	LAB_F080		; get WDxxxx status register
	AND	#$04			; mask 0000 0x00, track zero
	BEQ	LAB_F8E6		; do error exit if not track zero

	LDX	#$00			; flag ok
	RTS

LAB_F8E6
	LDX	#$06			; flag track zero error
LAB_F8E8
	LDA	#$FF			; deselect drives
	STA	LAB_00		; set drive select
	RTS

; seek to track

LAB_F8ED
	SMB3	LAB_01		; drive motor on
	JSR	LAB_F86A		; select current drive
	TXA				; copy error byte
	BNE	LAB_F8E8		; if error deselect drives and exit

	LDX	#$E0			; set delay count
LAB_F8F7
	DEX				; decrement count
	NOP				; waste ..
	NOP				; .. a few ..
	NOP				; .. cycles
	BNE	LAB_F8F7		; loop if not all done

	LDA	LAB_F081		; get WDxxxx track register
	CMP	LAB_F210		; compare with current track
	BEQ	LAB_F937		; exit if there

	LDA	LAB_F210		; get current track
	STA	LAB_F083		; write WDxxxx data register
	LDA	LAB_F214		; get disk command
	STA	LAB_F080		; set WDxxxx command register
	LDA	#$C8			; set for 200d counts
	STA	LAB_F220		; save countdown value
	JSR	LAB_F855		; initiate LAB_F220 countdown
	LDX	#$07			; flag timeout error
LAB_F91B
	LDA	LAB_F220		; get countdown value
	BEQ	LAB_F8E8		; if timed out deselect drives and exit

	LDA	LAB_F080		; get WDxxxx status register
	AND	#$01			; mask 0000 000x, busy
	BNE	LAB_F91B		; loop if busy

	RMB5	LAB_12		; clear counter interrupt enable
	LDA	LAB_F214		; get disk command
	AND	#$04			; mask verify track number bit
	BEQ	LAB_F937		; branch if not set

	LDA	LAB_F080		; get WDxxxx status register
	AND	#$10			; mask 000x 0000, seek error
	BNE	LAB_F93C		; branch if seek error

LAB_F937
	LDX	#$00			; flag no error
	RMB3	LAB_01		; drive motor off
	RTS

LAB_F93C
	LDX	#$15			; flag seek error
	RMB3	LAB_01		; drive motor off
	RTS

; write track

LAB_F941
	LDA	#<LAB_FB7E		; get write track exit code low byte
	STA	LAB_F242		; save PA0 -ve edge interrupt vector low byte
	LDA	#>LAB_FB7E		; get write track exit code high byte
	STA	LAB_F243		; save PA0 -ve edge interrupt vector high byte
	RMB1	LAB_10		; clear PA0 -ve edge detect flag
	SMB1	LAB_12		; set PA0 -ve edge detect enable

					; write IBM system 34 track, 256 bytes per sector

	LDX	#$50			; set count
	LDA	#$4E			; set byte
	CLI				; enable interrupts
	STY	LAB_F080		; set WDxxxx command register
LAB_F957
	BBR0	LAB_00,LAB_F957	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register
	DEX				; decrement count
	BNE	LAB_F957		; loop if more to do

	LDX	#$0C			; set count
	LDA	#$00			; set byte
LAB_F964
	BBR0	LAB_00,LAB_F964	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register
	DEX				; decrement count
	BNE	LAB_F964		; loop if more to do

	LDX	#$03			; set count
	LDA	#$F6			; set byte, writes $C2
LAB_F971
	BBR0	LAB_00,LAB_F971	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register
	DEX				; decrement count
	BNE	LAB_F971		; loop if more to do

	LDA	#$FC			; set byte, index mark
LAB_F97C
 	BBR0	LAB_00,LAB_F97C	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register

	LDX	#$32			; set count
	LDA	#$4E			; set byte
LAB_F986
	BBR0	LAB_00,LAB_F986	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register
	DEX				; decrement count
	BNE	LAB_F986		; loop if more to do

					; track header done, now do the sectors

	LDY	#$00			; clear count

LAB_F991
	LDX	#$0C			; set count
	LDA	#$00			; set byte
LAB_F995
	BBR0	LAB_00,LAB_F995	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register
	DEX				; decrement count
	BNE	LAB_F995		; loop if more to do

	LDX	#$03			; set count
	LDA	#$F5			; set byte, writes $A1
LAB_F9A2
	BBR0	LAB_00,LAB_F9A2	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register
	DEX				; decrement count
	BNE	LAB_F9A2		; loop if more to do

	LDA	#$FE			; set byte, ID address mark
LAB_F9AD
	BBR0	LAB_00,LAB_F9AD	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register

	LDA	LAB_F210		; get current track
LAB_F9B6
	BBR0	LAB_00,LAB_F9B6	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register

	LDA	LAB_F212		; get current side
LAB_F9BF
 	BBR0	LAB_00,LAB_F9BF	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register

	LDA	(LAB_41),Y		; get sector number from interleave table
LAB_F9C7
 	BBR0	LAB_00,LAB_F9C7	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register

	LDA	#$01			; set byte, sector length
LAB_F9CF
	BBR0	LAB_00,LAB_F9CF	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register

	LDA	#$F7			; set byte, write 2 CRC bytes
LAB_F9D7
	BBR0	LAB_00,LAB_F9D7	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register

	LDX	#$16			; set count
	LDA	#$4E			; set byte
LAB_F9E1
	BBR0	LAB_00,LAB_F9E1	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register
	DEX				; decrement count
	BNE	LAB_F9E1		; loop if more to do

	LDX	#$0C			; set count
	LDA	#$00			; set byte
LAB_F9EE
	BBR0	LAB_00,LAB_F9EE	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register
	DEX				; decrement count
	BNE	LAB_F9EE		; loop if more to do

	LDX	#$03			; set count
	LDA	#$F5			; set byte, writes $A1
LAB_F9FB
	BBR0	LAB_00,LAB_F9FB	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register
	DEX				; decrement count
	BNE	LAB_F9FB		; loop if more to do

	LDA	#$FB			; set byte, data address mark
LAB_FA06
	BBR0	LAB_00,LAB_FA06	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register

					; done the sector header, now clear the data block

	LDX	#$00			; set count, 256 bytes
	LDA	#$00			; set byte, clear data area
LAB_FA10
	BBR0	LAB_00,LAB_FA10	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register
	DEX				; decrement count
	BNE	LAB_FA10		; loop if more to do

					; done sector data, now do sector tail

	LDA	#$F7			; set byte, write 2 CRC bytes
LAB_FA1B
	BBR0	LAB_00,LAB_FA1B	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register

	LDX	#$36			; set count
	LDA	#$4E			; set byte
LAB_FA25
	BBR0	LAB_00,LAB_FA25	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register
	DEX				; decrement count
	BNE	LAB_FA25		; loop if more to do

	INY				; increment sector count
	CPY	#$10			; compare with max+1
	BEQ	LAB_FA36		; branch if all sectors done

	JMP	LAB_F991		; go do next sector

					; done sectors, now do track lead out
LAB_FA36
	LDA	#$4E			; set byte
LAB_FA38
	BBR0	LAB_00,LAB_FA38	; loop while WDxxxx data register full

	STA	LAB_F083		; save byte to WDxxxx data register
	JMP	LAB_FA36		; loop forever [1]


LAB_FA41
	.byte	$00,$08,$01,$09,$02,$0A,$03,$0B,$04,$0C,$05,$0D,$06,$0E,$07,$0F
					; sector numbers, interleave = 2
LAB_FA51
	.byte	$00,$0B,$06,$01,$0C,$07,$02,$0D,$08,$03,$0E,$09,$04,$0F,$0A,$05
					; sector numbers, interleave = 3
LAB_FA61
	.byte	$00,$04,$08,$0C,$01,$05,$09,$0D,$02,$06,$0A,$0E,$03,$07,$0B,$0F
					; sector numbers, interleave = 4
LAB_FA71
	.byte	$00,$01,$02,$03,$08,$09,$0A,$0B,$04,$05,$06,$07,$0C,$0D,$0E,$0F
					; sector numbers, interleave = 1
LAB_FA81
	.byte	<LAB_FA41		; sector numbers, interleave = 2
	.byte	<LAB_FA51		; sector numbers, interleave = 3
	.byte	<LAB_FA61		; sector numbers, interleave = 4
	.byte	<LAB_FA71		; sector numbers, interleave = 1

LAB_FA85
	.byte	>LAB_FA41		; sector numbers, interleave = 2
	.byte	>LAB_FA51		; sector numbers, interleave = 3
	.byte	>LAB_FA61		; sector numbers, interleave = 4
	.byte	>LAB_FA71		; sector numbers, interleave = 1

; format disk

LAB_FA89
	JSR	LAB_F8B9		; seek to track zero
	TXA				; copy error byte
	BNE	LAB_FAD3		; exit if error

	LDA	#$03			; mask 0000 00xx
	AND	LAB_F206		; get current interleave
	TAX				; copy to index
	LDA	LAB_FA81,X		; get interleave pattern pointer low byte
	STA	LAB_41		; set buffer pointer low byte
	LDA	LAB_FA85,X		; get interleave pattern pointer high byte
	STA	LAB_42		; set buffer pointer high byte
	LDA	#$00			; clear A
	STA	LAB_F210		; clear current track
LAB_FAA4
	LDA	#$00			; clear A
	STA	LAB_F212		; clear current side
	LDY	#$F0			; set command, write track, side 0
	JSR	LAB_F941		; write track
	TXA				; copy error byte
	BNE	LAB_FAD3		; exit if error

	INC	LAB_F212		; set side 1
	LDY	#$F2			; set command, write track, side 1
	JSR	LAB_F941		; write track
	TXA				; copy error byte
	BNE	LAB_FAD3		; exit if error

	INC	LAB_F210		; increment current track
	LDA	#$50			; set to max+1 = 80
	CMP	LAB_F210		; compare with current track
	BEQ	LAB_FAD3		; exit if there

	LDA	#$18			; seek, head load, 3ms step
	STA	LAB_F214		; save command
	JSR	LAB_F8ED		; seek to track
	TXA				; copy error byte
	BNE	LAB_FAD3		; exit if error

	BEQ	LAB_FAA4		; else loop for next track

LAB_FAD3
	LDA	#$00			; clear A
	STA	LAB_F212		; clear current side
	RTS

; read disk sector

LAB_FAD9
	LDA	LAB_F211		; get current sector
	STA	LAB_F082		; write WDxxxx sector register
	LDA	LAB_F210		; get current track
	STA	LAB_F081		; write WDxxxx track register
	LDX	#$88			; read sector, single, side 0, no side compare
	LDA	LAB_F212		; get current side
	BEQ	LAB_FAEE		; branch if side 0

	LDX	#$8A			; read sector, single, side 1, side compare
LAB_FAEE
	LDA	#<LAB_FBA4		; get read disk sector exit code low byte
	STA	LAB_F242		; save PA0 -ve edge interrupt vector low byte
	LDA	#>LAB_FBA4		; get read disk sector exit code high byte
	STA	LAB_F243		; save PA0 -ve edge interrupt vector high byte
	RMB1	LAB_10		; clear PA0 -ve edge detect flag
	SMB1	LAB_12		; set PA0 -ve edge detect enable
	LDY	#$00			; clear index
	STY	LAB_41		; clear buffer pointer low byte
	CLI				; enable interrupts
	STX	LAB_F080		; set WDxxxx command register
LAB_FB04
	BBR0	LAB_00,LAB_FB04	; loop while WDxxxx data register empty

	LDA	LAB_F083		; get WDxxxx data register
	STA	(LAB_41),Y		; save byte to buffer
	INY				; increment buffer index
	BNE	LAB_FB04		; loop if not all done

LAB_FB0F
	BBR0	LAB_00,LAB_FB0F	; loop while WDxxxx data register empty
	LDY	#$80			; set Nb
	BMI	LAB_FB0F		; loop forever [1]

; write disk sector

LAB_FB16
	LDA	LAB_F211		; get current sector
	STA	LAB_F082		; write WDxxxx sector register
	LDA	LAB_F210		; get current track
	STA	LAB_F081		; write WDxxxx track register
	LDX	#$A8			; write sector, single, side 0, no side compare
	LDA	LAB_F212		; get current side
	BEQ	LAB_FB2B		; branch if side 0

	LDX	#$AA			; write sector, single, side 1, side compare
LAB_FB2B
	LDA	#<LAB_FBD0		; get write disk sector exit code low byte
	STA	LAB_F242		; save PA0 -ve edge interrupt vector low byte
	LDA	#>LAB_FBD0		; get write disk sector exit code high byte
	STA	LAB_F243		; save PA0 -ve edge interrupt vector high byte
	RMB1	LAB_10		; clear PA0 -ve edge detect flag
	SMB1	LAB_12		; set PA0 -ve edge detect enable
	LDY	#$00			; clear index
	STY	LAB_41		; clear buffer pointer low byte
	CLI				; enable interrupts
	STX	LAB_F080		; set WDxxxx command register

LAB_FB41
	LDA	(LAB_41),Y		; get byte from buffer
LAB_FB43
	BBR0	LAB_00,LAB_FB43	; loop while WDxxxx data register full

	STA	LAB_F083		; write byte to WDxxxx data register
  	INY				; increment index
	BNE	LAB_FB41		; loop if more to do

LAB_FB4C
	BBR0	LAB_00,LAB_FB4C	; loop while WDxxxx data register full
	LDY	#$80			; set Nb
	BMI	LAB_FB4C		; loop forever [1]

; IRQ entry point

LAB_FB53
	PHA				; save A
	LDA	LAB_12		; get interrupt enable register
	AND	LAB_11		; mask interrupt flag register
	STA	LAB_10		; clear interrupt flag bits
	STA	LAB_40		; save interrupt flag bits
	BBS1	LAB_40,LAB_FB68	; branch if PA0 -ve edge interrupt

	BBS2	LAB_40,LAB_FB6B	; branch if internal interrupt

	BBS5	LAB_40,LAB_FB6E	; branch if counter interrupt

	JMP	(LAB_F240)		; do external interrupt vector

LAB_FB68
	JMP	(LAB_F242)		; do PA0 -ve edge interrupt vector

LAB_FB6B
	JMP	(LAB_F244)		; do internal interrupt vector

LAB_FB6E
	JMP	(LAB_F246)		; do counter interrupt vector

; decrement delay count and restart counter

LAB_FB71
	LDA	#$4E			; set counter high byte, 5ms
	STA	LAB_19		; save counter high byte and start counter
	DEC	LAB_F220		; decrement count
	BNE	LAB_FB7C		; branch if not all done yet

	RMB5	LAB_12		; clear counter interrupt enable
LAB_FB7C
	PLA				; restore A
	RTI

; write track exit code

LAB_FB7E
	RMB1	LAB_12		; clear PA0 -ve edge detect enable
	RMB1	LAB_10		; clear PA0 -ve edge detect flag
	PLA				; dump interrupt status byte
	PLA				; dump interrupt return address low byte
	PLA				; dump interrupt return address high byte
	PLA				; dump saved Accumulator
	LDA	LAB_F080		; get WDxxxx status register
	STA	LAB_40		; save WDxxxx status register
	LDX	#$04			; flag not ready error
	BBS7	LAB_40,LAB_FBA2	; branch if not ready

	LDX	#$03			; flag write protect error
	BBS6	LAB_40,LAB_FBA2	; branch if write protected

	LDX	#$08			; flag write error
	BBS2	LAB_40,LAB_FBA2	; branch if data error

	LDX	#$1A			; flag not 16 sectors written
	CPY	#$10			; compare sector count with 16
	BNE	LAB_FBA2		; branch if not 16 sectors written

	LDX	#$00			; flag no error
LAB_FBA2
	CLI				; enable interrupts
	RTS

; read disk sector exit code

LAB_FBA4
	RMB1	LAB_12		; clear PA0 -ve edge detect enable
	RMB1	LAB_10		; clear PA0 -ve edge detect flag
	PLA				; dump interrupt status byte
	PLA				; dump interrupt return address low byte
	PLA				; dump interrupt return address high byte
	PLA				; dump saved Accumulator
	LDA	LAB_F080		; get WDxxxx status register
	STA	LAB_40		; save WDxxxx status register
	LDX	#$04			; flag not ready error
	BBS7	LAB_40,LAB_FBCC	; branch if not ready

	LDX	#$14			; flag sector error
	BBS4	LAB_40,LAB_FBCC	; branch if sector error

	LDX	#$11			; flag CRC error
	BBS3	LAB_40,LAB_FBCC	; branch if CRC error

	LDX	#$09			; flag read error
	BBS2	LAB_40,LAB_FBCC	; branch if data error

 	LDX	#$0A			; flag not 256 bytes read
	TYA				; copy read index
	BNE	LAB_FBCC		; branch if not 256 bytes read

	LDX	#$00			; flag no error
LAB_FBCC
	CLI				; enable interrupts
	RMB4	LAB_01		; only accessed here ??
	RTS

; write disk sector exit code

LAB_FBD0
	RMB1	LAB_12		; clear PA0 -ve edge detect enable
	RMB1	LAB_10		; clear PA0 -ve edge detect flag
	PLA				; dump interrupt status byte
	PLA				; dump interrupt return address low byte
	PLA				; dump interrupt return address high byte
	PLA				; dump saved Accumulator
	LDA	LAB_F080		; get WDxxxx status register
	STA	LAB_40		; save WDxxxx status register
	LDX	#$04			; flag not ready error
	BBS7	LAB_40,LAB_FBF8	; branch if not ready

	LDX	#$14			; flag sector error
	BBS4	LAB_40,LAB_FBF8	; branch if sector error

	LDX	#$03			; flag write protect error
	BBS6	LAB_40,LAB_FBF8	; branch if write protected

	LDX	#$08			; flag write error
	BBS2	LAB_40,LAB_FBF8	; branch if data error

	LDX	#$0B			; flag not 256 bytes written
	TYA				; copy write index
	BNE	LAB_FBF8		; branch if not 256 bytes written

	LDX	#$00			; flag no error
LAB_FBF8
	CLI				; enable interrupts
	RTS

; wait for byte from host

LAB_FBFA
	LDA	#$C0			; busy, Rx, command
	STA	LAB_1E		; set host status register
	LDA	#$00			; clear A
	STA	LAB_F207		; clear report byte
	BBR0	LAB_1E,LAB_FC08	; branch if host buffer empty

	LDA	LAB_1C		; read byte from host
LAB_FC08
	LDA	#$C8			; set for 200d counts
	STA	LAB_F220		; save countdown value
	JSR	LAB_F855		; initiate LAB_F220 countdown
	CLI				; enable interrupts
	LDA	#$40			; ready, Rx, command
	STA	LAB_1E		; set host status register
LAB_FC15
	LDA	LAB_F220		; get countdown value
	BNE	LAB_FC21		; branch if not timed out

	INC	LAB_F220		; increment countdown value
	LDA	#$FF			; deselect drives
	STA	LAB_00		; set drive select
LAB_FC21
	BBR0	LAB_1E,LAB_FC15	; loop if host Rx buffer empty

	RMB5	LAB_12		; clear counter interrupt enable
	LDA	#$C0			; busy, Rx, command
	STA	LAB_1E		; set host status register
	LDY	LAB_1C		; read byte from host
	TYA				; copy received byte to A
	AND	#$07			; mask 0000 0xxx, drive number
	STA	LAB_F201		; save command drive number
	TYA				; copy received byte to A
	AND	#$F8			; mask xxxx x000, command only
	LDY	#$0F			; index to last command
LAB_FC37
	CMP	LAB_FC8E,Y		; compare with byte from command table
	BEQ	LAB_FC5B		; branch if match

	DEY				; decrement index
	BPL	LAB_FC37		; loop if more to do

	LDA	#$20			; flag unknown command error
	STA	LAB_F207		; set report byte

; send report byte to host

LAB_FC44
	LDA	#$80			; busy, Tx, command
	STA	LAB_1E		; set host status register
	LDA	LAB_F207		; get report byte
	STA	LAB_1C		; write byte to host
	LDA	#$00			; ready, Tx, command
	STA	LAB_1E		; set host status register
LAB_FC51
	BBS1	LAB_1E,LAB_FC51	; loop if host Tx buffer full

	LDA	#$C0			; busy, Rx, command
	STA	LAB_1E		; set host status register
	RMB1	LAB_01		; only accessed here ??
	RTS
					; command byte matched
LAB_FC5B
	LDA	LAB_F201		; get command drive number
	CMP	LAB_F213		; compare with maximum drive+1
	BCC	LAB_FC6A		; branch if less

	LDA	#$21			; flag command parameter error
	STA	LAB_F207		; set report byte
	BNE	LAB_FC44		; send report byte to host and return

LAB_FC6A
	STY	LAB_F200		; save command index
	LDA	#$40			; ready, Rx, command
	STA	LAB_1E		; set host status register
LAB_FC71
	BBR0	LAB_1E,LAB_FC71	; loop if host Rx buffer empty

	LDA	LAB_1C		; read byte from host
	STA	LAB_F202		; save command byte 1
LAB_FC79
	BBR0	LAB_1E,LAB_FC79	; loop if host Rx buffer empty

	LDA	LAB_1C		; read byte from host
	STA	LAB_F203		; save command byte 2
LAB_FC81
	BBR0	LAB_1E,LAB_FC81	; loop if host Rx buffer empty

	LDA	#$C0			; busy, Rx, command
	STA	LAB_1E		; set host status register
	LDA	LAB_1C		; read byte from host
	STA	LAB_F204		; save command byte 3
	RTS

; command bytes, $FF will never match received command

LAB_FC8E
	.byte	$FF			; no command
	.byte	$FF			; no command
	.byte	$FF			; no command
	.byte	$FF			; no command
	.byte	$FF			; no command
	.byte	$90			; error $20
	.byte	$88			; error $20
	.byte	$80			; seek to track zero
	.byte	$FF			; no command
	.byte	$FF			; no command
	.byte	$FF			; no command
	.byte	$FF			; no command
	.byte	$98			; format disk
	.byte	$D0			; error $20
	.byte	$C8			; write sector
	.byte	$C0			; read sector

; read 256 bytes from host

LAB_FC9E
	LDY	#$00			; clear Y
	STY	LAB_41		; clear buffer pointer low byte
	LDA	#$E0			; busy, Rx, data
	STA	LAB_1E		; set host status register
	LDA	#$60			; ready, Rx, data
	STA	LAB_1E		; set host status register
LAB_FCAA
	BBR0	LAB_1E,LAB_FCAA	; loop if host Rx buffer empty

	CPY	#$FF			; compare index with $FF
	BNE	LAB_FCB5		; branch if not full yet

	LDA	#$E0			; busy, Rx, data
	STA	LAB_1E		; set host status register
LAB_FCB5
	LDA	LAB_1C		; read byte from host
	STA	(LAB_41),Y		; save to buffer
	INY				; increment index
	BNE	LAB_FCAA		; loop if not all done

	RTS

; write 256 bytes to host

LAB_FCBD
	LDA	#$A0			; busy, Tx, data
	STA	LAB_1E		; set host status register
	LDY	#$00			; clear index
	STY	LAB_41		; clear buffer pointer low byte
	LDA	(LAB_41),Y		; get byte from buffer
	STA	LAB_1C		; write byte to host
	LDA	#$20			; ready, Tx, data
	STA	LAB_1E		; set host status register
	BPL	LAB_FCD4		; branch into loop, branch always

LAB_FCCF
	INY				; increment index
	LDA	(LAB_41),Y		; get byte from buffer
	STA	LAB_1C		; write byte to host
LAB_FCD4
	BBS1	LAB_1E,LAB_FCD4	; loop if host Tx buffer full

	CPY	#$FF			; compare index with end
	BNE	LAB_FCCF		; loop if not all done

	LDA	#$A0			; busy, Tx, data
	STA	LAB_1E		; set host status register
	RTS

; evaluate command bytes

; byte 1		track high nibble		$00 to $04, side 0
;							$05 to $09, side 1

; byte 2		$.X sector
;			$X. track low nibble

LAB_FCE0
	LDA	#$21			; flag command parameter error
	STA	LAB_F207		; set report byte
	LDA	#$00			; clear A
	STA	LAB_F212		; clear current side
	LDY	#$04			; set max+1 for first compare
	CPY	LAB_F202		; compare with command byte 1
	BCS	LAB_FD04		; branch if command byte 1 is <= $04

	LDY	#$09			; set max for second compare
	CPY	LAB_F202		; compare with command byte 1
	BCC	LAB_FD42		; exit if command byte 1 is > $09

	LDA	LAB_F202		; get command byte 1
	SEC				; set carry for subtract, not needed carry is set
	SBC	#$05			; convert side 1 to side 0
	STA	LAB_F202		; save command byte 1
	INC	LAB_F212		; increment current side to side 1
LAB_FD04
	LDX	LAB_F204		; get command byte 3
	STX	LAB_F215		; copy it, this byte is never read
	DEX				; decrement it
	TXA				; copy to A
	CLC				; clear carry for add
	ADC	LAB_F203		; add command byte 2
	BCC	LAB_FD17		; branch if no carry, is this a checksum ??

	CPY	LAB_F202		; compare max with command byte 1
	BEQ	LAB_FD42		; exit if command byte 1 is = max ??

LAB_FD17
	LDA	LAB_F201		; get command drive number
	STA	LAB_F20C		; save current drive number
	LDA	#$0F			; mask 0000 xxxx
	AND	LAB_F203		; get command byte 2
	STA	LAB_F211		; set current sector

	LDX	#$04			; set count
	LDY	LAB_F202		; get command byte 1
	LDA	LAB_F203		; get command byte 2
LAB_FD2D
	ROR	LAB_F202		; shift command byte 1
	ROR				; shift command byte 2
	DEX				; decrement count
	BNE	LAB_FD2D		; loop if more to do

	STA	LAB_F210		; set current track
	STY	LAB_F202		; restore command byte 1
	LDA	#$00			; clear A
	NOP				; waste ..
	NOP				; .. a bit ..
	NOP				; .. of space
	STA	LAB_F207		; clear report byte
LAB_FD42
	RTS

; RESET and NMI entry

LAB_FD43
	SEI				; disable interrupts
	LDA	#$60			; port B all outputs
					; INT = INT
					; bus = 65xx
					; counter = interval timer
	STA	LAB_14		; set mode control register
	CLD				; clear decimal mode
	LDX	#$7F			; top of internal RAM
	TXS				; set stack pointer to $007F
	LDA	#$C0			; busy, Rx, command
	STA	LAB_1E		; set host status register
	LDA	#$FF			; all inputs
	STA	LAB_02		; set port C
	LDA	#$FF			; deselect drives
	STA	LAB_00		; set drive select
	LDA	#$00			; clear A
	STA	LAB_12		; clear interrupt enable register
	STA	LAB_01		; motor off
	STA	LAB_F216		; clear active drive
	STA	LAB_F217		; clear other drive track number
	STA	LAB_F212		; clear current side
	STA	LAB_F20C		; clear current drive number
	JSR	LAB_F800		; set up vectors
	LDA	#$02			; set max drive+1
	STA	LAB_F213		; save max drive+1
	CLI				; enable interrupts
	JSR	LAB_F8B9		; seek to track zero, drive 0
	INC	LAB_F20C		; increment current drive number
	JSR	LAB_F8B9		; seek to track zero, drive 1
LAB_FD7C
	JSR	LAB_FBFA		; wait for byte from host
	LDA	LAB_F207		; get report byte
	BNE	LAB_FD7C		; loop if was error

	LDX	LAB_F200		; get command index
	LDA	LAB_FE13,X		; get command vector low byte
	STA	LAB_F248		; save as command vector address low byte
	LDA	LAB_FE23,X		; get command vector high byte
	STA	LAB_F249		; save as command vector address high byte
	JSR	LAB_FD99		; execute command vector as subroutine
	JMP	LAB_FD7C		; loop for next command

LAB_FD99
	JMP	(LAB_F248)		; execute command vector

; seek to track zero and report

LAB_FD9C
	LDA	LAB_F201		; get command drive number
	STA	LAB_F20C		; save current drive number
	JSR	LAB_F8B9		; seek to track zero
	STX	LAB_F207		; save error byte as report byte
	JSR	LAB_FC44		; send report byte to host
	RTS

; format disk and report

LAB_FDAC
	LDA	LAB_F201		; get command drive number
	STA	LAB_F20C		; save current drive number
	LDA	LAB_F204		; command byte 3
	STA	LAB_F206		; set current interleave
	JSR	LAB_FA89		; format disk
	STX	LAB_F207		; save error byte as report byte
	JSR	LAB_FC44		; send report byte to host
	RTS

; read sector

LAB_FDC2
	JSR	LAB_FCE0		; evaluate command bytes
	LDA	LAB_F207		; get report byte
	BNE	LAB_FDE8		; report and exit if error

	LDA	#$1C			; seek, head load, verify track number, 3ms step
	STA	LAB_F214		; save command
	JSR	LAB_F8ED		; seek to track
	STX	LAB_F207		; save error byte as report byte
	TXA				; copy error byte
	BNE	LAB_FDE8		; report and exit if error

	LDA	#>LAB_F300		; set buffer pointer high byte
	STA	LAB_42		; save buffer pointer high byte
	JSR	LAB_FAD9		; read disk sector
	STX	LAB_F207		; save error byte as report byte
	TXA				; copy error byte
	BNE	LAB_FDE8		; report and exit if error

	JSR	LAB_FCBD		; write 256 bytes to host
LAB_FDE8
	JSR	LAB_FC44		; send report byte to host
	RTS

; write sector

LAB_FDEC
	JSR	LAB_FCE0		; evaluate command bytes
	LDA	LAB_F207		; get report byte
	BNE	LAB_FE0F		; report and exit if error

	LDA	#>LAB_F300		; set buffer pointer high byte
	STA	LAB_42		; save buffer pointer high byte
	JSR	LAB_FC9E		; read 256 bytes from host
	LDA	#$1C			; seek, head load, verify track number, 3ms step
	STA	LAB_F214		; save command
	JSR	LAB_F8ED		; seek to track
	STX	LAB_F207		; save error byte as report byte
	TXA				; copy error byte
	BNE	LAB_FE0F		; report and exit if error

	JSR	LAB_FB16		; write disk sector
	STX	LAB_F207		; save error byte as report byte
LAB_FE0F
	JSR	LAB_FC44		; send report byte to host
	RTS

; command vector table low bytes

LAB_FE13
	.byte	<LAB_FE33		; $FF set report byte = $20 and send it
	.byte	<LAB_FE33		; $FF set report byte = $20 and send it
 	.byte	<LAB_FE33		; $FF set report byte = $20 and send it
 	.byte	<LAB_FE33		; $FF set report byte = $20 and send it
	.byte	<LAB_FE33		; $FF set report byte = $20 and send it
	.byte	<LAB_FE33		; $90 set report byte = $20 and send it
	.byte	<LAB_FE33		; $88 set report byte = $20 and send it
	.byte	<LAB_FD9C		; $80 seek to track zero and report
	.byte	<LAB_FE33		; $FF set report byte = $20 and send it
	.byte	<LAB_FE33		; $FF set report byte = $20 and send it
	.byte	<LAB_FE33		; $FF set report byte = $20 and send it
	.byte	<LAB_FE33		; $FF set report byte = $20 and send it
	.byte	<LAB_FDAC		; $98 format disk and report
 	.byte	<LAB_FE33		; $D0 set report byte = $20 and send it
 	.byte	<LAB_FDEC		; $C8 write sector
	.byte	<LAB_FDC2		; $C0 read sector

; command vector table high bytes

LAB_FE23
	.byte	>LAB_FE33		; set report byte = $20 and send it
	.byte	>LAB_FE33		; set report byte = $20 and send it
 	.byte	>LAB_FE33		; set report byte = $20 and send it
 	.byte	>LAB_FE33		; set report byte = $20 and send it
	.byte	>LAB_FE33		; set report byte = $20 and send it
	.byte	>LAB_FE33		; set report byte = $20 and send it
	.byte	>LAB_FE33		; set report byte = $20 and send it
	.byte	>LAB_FD9C		; seek to track zero and report
	.byte	>LAB_FE33		; set report byte = $20 and send it
	.byte	>LAB_FE33		; set report byte = $20 and send it
	.byte	>LAB_FE33		; set report byte = $20 and send it
	.byte	>LAB_FE33		; set report byte = $20 and send it
	.byte	>LAB_FDAC		; format disk and report
 	.byte	>LAB_FE33		; set report byte = $20 and send it
 	.byte	>LAB_FDEC		; write sector
	.byte	>LAB_FDC2		; read sector

; set report byte = $20 and send it

LAB_FE33
	LDA	#$20			; flag unknown command error
	STX	LAB_F207		; set report byte
	JSR	LAB_FC44		; send report byte to host
	RTS

; here on is unreferenced

;LAB_FE3C
	.byte	$52,$9E,$8D,$F2
	.byte	$03,$A9,$06,$D0,$05,$AD,$62,$AA,$F0,$06,$8D,$5F,$AA,$4C,$80,$A1
	.byte	$60,$4C,$BF,$9D,$4C,$84,$9D,$4C,$FD,$AA,$4C,$B5,$B7,$AD,$0F,$9D
	.byte	$AC,$0E,$9D,$60,$AD,$C2,$AA,$AC,$C1,$AA,$60,$4C,$51,$A8,$EA,$EA
	.byte	$4C,$59,$FA,$4C,$65,$FF,$4C,$58,$FF,$4C,$65,$FF,$4C,$65,$FF,$65
	.byte	$FF,$20,$D1,$9E,$AD,$51,$AA,$F0,$15,$48,$AD,$5C,$AA,$91,$28,$68
	.byte	$30,$03,$4C,$26,$A6,$20,$EA,$9D,$A4,$24,$A9,$60,$91,$28,$AD,$B3
	.byte	$AA,$F0,$03,$20,$82,$A6,$A9,$03,$8D,$52,$AA,$20,$BA,$9F,$20,$BA
	.byte	$9E,$8D,$5C,$AA,$8E,$5A,$AA,$4C,$B3,$9F,$6C,$38,$00,$20,$D1,$9E
	.byte	$AD,$52,$AA,$0A,$AA,$BD,$11,$9D,$48,$BD,$10,$9D,$48,$AD,$5C,$AA
	.byte	$60,$8D,$5C,$AA,$8E,$5A,$AA,$8C,$5B,$AA,$BA,$E8,$E8,$8E,$59,$AA
	.byte	$A2,$03,$BD,$53,$AA,$95,$36,$CA,$10,$F8,$60,$AE,$B7,$AA,$F0,$03
	.byte	$4C,$78,$9F,$AE,$51,$AA,$F0,$08,$C9,$BF,$F0,$75,$C5,$33,$F0,$27
	.byte	$A2,$02,$8E,$52,$AA,$CD,$B2,$AA,$D0,$19,$CA,$8E,$52,$AA,$CA,$8E
	.byte	$5D,$AA,$AE,$5D,$AA,$9D,$00,$02,$E8,$8E,$5D,$AA,$C9,$8D,$D0,$75
	.byte	$4C,$CD,$9F,$C9,$8D,$D0,$7D,$A2,$00,$8E,$52,$AA,$4C,$A4,$9F,$A2
	.byte	$00,$8E,$52,$AA,$C9,$8D,$F0,$07,$AD,$B3,$AA,$F0,$67,$D0,$5E,$48
	.byte	$38,$AD,$B3,$AA,$D0,$03,$20,$5E,$A6,$68,$90,$EC,$AE,$5A,$AA,$4C
	.byte	$15,$9F,$C9,$8D,$D0,$05,$A9,$05,$8D,$52,$AA,$20,$0E,$A6,$4C,$99
	.byte	$9F,$CD,$B2,$AA,$F0,$85,$C9,$8A,$F0,$F1,$A2,$04,$8E,$52,$AA,$D0
	.byte	$E1,$A9,$00,$8D,$52,$AA,$F0,$25,$A9,$00,$8D,$B7,$AA,$20,$51,$A8
	.byte	$4C,$DC,$A4,$AD,$00,$02,$CD,$B2,$AA,$F0,$0A,$A9,$8D,$8D,$00,$02
	.byte	$A2,$00,$8E,$5A,$AA,$A9,$40,$D0,$06,$A9,$10,$D0,$02,$A9,$20,$2D
	.byte	$5E,$AA,$F0,$0F,$20,$BA,$9F,$20,$C5,$9F,$8D,$5C,$AA,$8C,$5B,$AA
	.byte	$8E,$5A,$AA,$20,$51,$A8,$AE,$59,$AA,$9A,$AD,$5C,$AA,$AC,$5B,$AA
	.byte	$AE,$5A,$AA,$38,$60,$6C,$36,$00,$A9,$8D,$4C,$C5,$9F,$A0,$FF,$8C
	.byte	$5F,$AA,$C8,$8C,$62,$AA,$EE,$5F,$AA,$A2,$00,$08,$BD,$00,$02,$CD
	.byte	$B2,$AA,$D0,$01,$E8,$8E,$5D,$AA,$20,$A4,$A1,$29,$7F,$59,$84,$A8
	.byte	$C8,$0A,$F0,$02,$68,$08,$90,$F0,$28,$F0

; system vectors

	.ORG	$FFFA

	.word	LAB_FD43		; NMI vector
 	.word	LAB_FD43		; RESET vector
	.word	LAB_FB53		; IRQ vector

	.END
